@@ -5,7 +5,7 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Union
 
 if TYPE_CHECKING:
     from requests import Response
@@ -231,60 +231,37 @@ class Api:
 
     def __init__(
         self,
-        hostname,
-        port=None,
-        url=None,
-        protocol=None,
-        ignore_ssl_errors=False,
-        username=None,
-        password=None,
+        hostname: str = None,
+        port: int = None,
+        url: str = None,
+        protocol: str = None,
+        ignore_ssl_errors: bool = False,
+        username: str = None,
+        password: str = None,
     ):
         """
 
         :param str hostname:
-        :param int port:
-        :param str protocol:
-        :param bool ignore_ssl_errors:
-        :param str username:
-        :param str password:
+        :param int port: The endpoint port
+        :param str protocol: http or https
+        :param bool ignore_ssl_errors: Ignore SSL errors, for self-signed endpoints. Use at own risks
+        :param str username: Username used for basic auth
+        :param str password: Password used for basic auth
         """
-        self.hostname = hostname
-        self.protocol = protocol if protocol else "http"
-        self.verify_ssl = not ignore_ssl_errors
-        self.port = port if port else 8083
+        if (username and not password) or (password and not username):
+            raise ValueError("You must specify both username and password")
         self.username = username
         self.password = password
+        self.hostname = hostname
 
-        if self.protocol not in ["http", "https"]:
-            raise ValueError("protocol must be one of", ["http", "https"])
-        if (self.port < 0) or (self.port > (2**16)):
-            raise ValueError(
-                f"Port {self.port} is not valid. Must be between 0 and {((2 ** 16) - 1)}"
-            )
-        if self.username and not self.password or self.password and not self.username:
-            raise ValueError("You must specify both username and password")
-        if self.verify_ssl is True and self.protocol == "http":
-            print("No SSL needed for HTTP without TLS. Disabling")
-            self.verify_ssl = False
-        if url:
-            self.url = url
-            if not re.match(r"(http://|https://)", self.url):
-                print(f"URL Does not contain a protocol. Using default {self.protocol}")
-                self.url = f"{self.protocol}://{self.url}"
-            print("URL Defined from parameter. Skipping hostname:port parameters")
-        elif (self.port == 80 and protocol == "http") or (
-            self.port == 443 and self.protocol == "https"
-        ):
-            self.url = f"{self.protocol}://{self.hostname}"
-        else:
-            self.url = f"{self.protocol}://{self.hostname}:{self.port}"
-
-        self.auth = (
-            HTTPBasicAuth(self.username, self.password)
-            if self.username and self.password
-            else None
-        )
+        self._url = None
+        self._port = None
+        self._protocol = "http"
+        self._ignore_ssl_errors = ignore_ssl_errors
 
+        self.protocol = protocol
+        self.port = port
+        self.url = url
         self.headers = {
             "Content-type": "application/json",
             "Accept": "application/json",
@@ -293,13 +270,97 @@ class Api:
     def __repr__(self):
         return self.url
 
+    @property
+    def verify_ssl(self) -> bool:
+        if not self._ignore_ssl_errors and self.protocol == "http":
+            return False
+        return not self._ignore_ssl_errors
+
+    @property
+    def basic_auth(self) -> Union[HTTPBasicAuth, None]:
+        """Returns basic auth information. If both the username and password are not set, raises AttributeError"""
+        if self.username and self.password:
+            return HTTPBasicAuth(self.username, self.password)
+        if (self.username and not self.password) or (
+            self.password and not self.username
+        ):
+            raise AttributeError("You must specify both username and password")
+        return None
+
+    @property
+    def url(self) -> str:
+        if self._url:
+            return self._url
+        elif self.hostname:
+            if (self.port == 80 and self.protocol == "http") or (
+                self.port == 443 and self.protocol == "https"
+            ):
+                return f"{self.protocol}://{self.hostname}"
+            else:
+                return f"{self.protocol}://{self.hostname}:{self.port}"
+        else:
+            raise AttributeError(
+                "Unable to form a URL", self._url, self.hostname, self.port
+            )
+
+    @url.setter
+    def url(self, value: Union[str, None]):
+        if value is None:
+            return
+        if re.match(r"^https://(.*)$", value):
+            self.protocol = "https"
+        elif not re.match(r"(http://|https://)", value):
+            print(f"URL Does not contain a protocol. Using default {self.protocol}")
+            value = f"{self.protocol}://{value}"
+        self._url = value
+
+    @property
+    def protocol(self) -> str:
+        if self._protocol:
+            return self._protocol
+        return "http"
+
+    @protocol.setter
+    def protocol(self, value: Union[str, None]) -> None:
+        if value is None:
+            return
+        valid_values: list = ["http", "https", "HTTP", "HTTPS"]
+        if value not in valid_values:
+            raise ValueError("protocol must be one of", valid_values, "got", value)
+        self._protocol = value.lower()
+
+    @property
+    def port(self) -> int:
+        if self._port:
+            return self._port
+        return 80
+
+    @port.setter
+    def port(self, value: Union[int, None]) -> None:
+        if value is None and self.protocol == "http":
+            value = 80
+        elif value is None and self.protocol == "https":
+            value = 443
+        elif isinstance(value, str):
+            value = int(value)
+
+        if (value < 0) or (value > (2**16)):
+            raise ValueError(
+                f"Port {self.port} is not valid. Must be between 0 and {((2 ** 16) - 1)}"
+            )
+        self._port = value
+
     @evaluate_api_return
     def get_raw(self, query_path, **kwargs) -> Response:
         if not query_path.startswith(r"/"):
             query_path = f"/{query_path}"
         url = f"{self.url}{query_path}"
         req = get(
-            url, auth=self.auth, headers=self.headers, verify=self.verify_ssl, **kwargs
+            url,
+            auth=self.basic_auth,
+            headers=self.headers,
+            verify=self.verify_ssl,
+            **kwargs,
         )
         return req
 
@@ -314,7 +375,7 @@ class Api:
         url = f"{self.url}{query_path}"
         req = post(
             url,
-            auth=self.auth,
+            auth=self.basic_auth,
             headers=self.headers,
             verify=self.verify_ssl,
             **kwargs,
@@ -332,7 +393,7 @@ class Api:
         url = f"{self.url}{query_path}"
         req = put(
             url,
-            auth=self.auth,
+            auth=self.basic_auth,
             headers=self.headers,
             verify=self.verify_ssl,
             **kwargs,
@@ -349,7 +410,11 @@ class Api:
             query_path = f"/{query_path}"
         url = f"{self.url}{query_path}"
         req = delete(
-            url, auth=self.auth, headers=self.headers, verify=self.verify_ssl, **kwargs
+            url,
+            auth=self.basic_auth,
+            headers=self.headers,
+            verify=self.verify_ssl,
+            **kwargs,
         )
         return req
 
